local Grid2StatusAuraGroup = Grid2:NewModule("StatusAuraGroup")

local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

--{{{  Status Creation

-- UpdateState for all Auras in the Group
local function status_UpdateState(self, unit, auraName, iconTexture, count, duration, expiration)
	if (self.auraNames[auraName]) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

-- UpdateState for only my Auras in the Group
local function status_UpdateStateMine(self, unit, auraName, iconTexture, count, duration, expiration, isMine)
	if (self.auraNames[auraName] and isMine) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

--}}}


-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	if (dblData) then
		upgrade = DBL:LoadOptions("Grid2StatusAuraGroupOptions", dblData, nil, "account", 2, dblData.classKey, 1) or upgrade
	else
		DBL:LoadOptions("Grid2StatusAuraGroupOptions")
		upgrade = true
	end
	
	return upgrade
end


-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)
	
	local Grid2StatusAuraGroupOptions = Grid2Options.plugins["Grid2StatusAuraGroupOptions"]
	if (Grid2StatusAuraGroupOptions) then
		flatten = DBL:UpgradeDefaults("Grid2StatusAuraGroupOptions", dblData, Grid2StatusAuraGroupOptions.UpgradeDefaults, "account", 2, dblData.classKey, 1) or flatten
	end
	return flatten
end


local function CreateAuraGroup(status, dbx)
	_, _, status.iconTexture = GetSpellInfo(dbx.spellName)

	local auraNames = {}
	status.auraNames = auraNames
	for index, spellName in pairs(dbx.auraNames) do
		if (type(spellName) == "number") then
			spellName = GetSpellInfo(spellName)
		end
		assert(type(spellName) == "string", "bad spellName" .. tostring(dbx.auraNames[index]))
		auraNames[spellName] = true
	end

	status:UpdateProfileData()
	return status
end


local statusTypes = { "color", "icon", "percent" }

-- Buff type group
local function CreateAuraGroupBuff(baseKey, dbx)
	local status = Grid2.CreateBuff(baseKey, dbx, statusTypes)

	CreateAuraGroup(status, dbx)
	status.UpdateState = dbx.mine and status_UpdateStateMine or status_UpdateState

	return status
end

-- Debuff type group
local function CreateAuraGroupDebuff(baseKey, dbx)
	local status = Grid2.CreateDebuff(baseKey, dbx, statusTypes)

	CreateAuraGroup(status, dbx)
	status.UpdateState = status_UpdateState

	return status
end

Grid2.setupFunc["auraGroupBuff"] = CreateAuraGroupBuff
Grid2.setupFunc["auraGroupDebuff"] = CreateAuraGroupDebuff
